@@ -34,7 +34,8 @@ def mct_config_str(config):
         'star_reference': 'required',
         'gtf_path': 'required',
         'feature_type': 'gene',
-        'id_type': 'gene_id'
+        'id_type': 'gene_id',
+        'nome_flag_str': 'required'
     }
 
     typed_config = {}
