@@ -60,6 +60,9 @@ def summarize_select_dna_reads(output_dir,
             continue
 
         cell_id = path.name.split('.')[0]
+        if cell_id.endswith('-R1') or cell_id.endswith('-R2'):
+            # select DNA preformed in R1 R2 separately:
+            cell_id = cell_id[:-3]
         _df['cell_id'] = cell_id
         _df['mc_rate_max_threshold'] = mc_rate_max_threshold
         _df['cov_min_threshold'] = cov_min_threshold
@@ -68,7 +71,7 @@ def summarize_select_dna_reads(output_dir,
 
     selected_reads = total_stats_df[
         (total_stats_df['cov'] >= cov_min_threshold)
-        & (total_stats_df['mc_rate'] < mc_rate_max_threshold)]
+        & (total_stats_df['mc_frac'] < mc_rate_max_threshold)]
 
     selected_reads = selected_reads.groupby('cell_id')['count'].sum()
     selected_ratio = selected_reads / total_stats_df.groupby('cell_id')['count'].sum()
